@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -23,8 +23,9 @@ import os
 import sys
 import multiprocessing
 from io import open
+import re
 
-from glances.compat import ConfigParser, NoOptionError
+from glances.compat import ConfigParser, NoOptionError, system_exec
 from glances.globals import BSD, LINUX, MACOS, SUNOS, WINDOWS
 from glances.logger import logger
 
@@ -42,7 +43,10 @@ def user_config_dir():
         path = os.path.expanduser('~/Library/Application Support')
     else:
         path = os.environ.get('XDG_CONFIG_HOME') or os.path.expanduser('~/.config')
-    path = os.path.join(path, 'glances')
+    if path is None:
+        path = ''
+    else:
+        path = os.path.join(path, 'glances')
 
     return path
 
@@ -52,10 +56,11 @@ def user_cache_dir():
 
     - Linux, *BSD, SunOS: ~/.cache/glances
     - macOS: ~/Library/Caches/glances
-    - Windows: %LOCALAPPDATA%\glances\cache
+    - Windows: {%LOCALAPPDATA%,%APPDATA%}\glances\cache
     """
     if WINDOWS:
-        path = os.path.join(os.environ.get('LOCALAPPDATA'), 'glances', 'cache')
+        path = os.path.join(os.environ.get('LOCALAPPDATA') or os.environ.get('APPDATA'),
+                            'glances', 'cache')
     elif MACOS:
         path = os.path.expanduser('~/Library/Caches/glances')
     else:
@@ -78,7 +83,10 @@ def system_config_dir():
         path = '/usr/local/etc'
     else:
         path = os.environ.get('APPDATA')
-    path = os.path.join(path, 'glances')
+    if path is None:
+        path = ''
+    else:
+        path = os.path.join(path, 'glances')
 
     return path
 
@@ -96,6 +104,9 @@ class Config(object):
         self.config_filename = 'glances.conf'
         self._loaded_config_file = None
 
+        # Re patern for optimize research of `foo`
+        self.re_pattern = re.compile('(\`.+?\`)')
+
         self.parser = ConfigParser()
         self.read()
 
@@ -128,6 +139,7 @@ class Config(object):
     def read(self):
         """Read the config file, if it exists. Using defaults otherwise."""
         for config_file in self.config_file_paths():
+            logger.info('Search glances.conf file in {}'.format(config_file))
             if os.path.exists(config_file):
                 try:
                     with open(config_file, encoding='utf-8') as f:
@@ -135,7 +147,7 @@ class Config(object):
                         self.parser.read(f)
                     logger.info("Read configuration file '{}'".format(config_file))
                 except UnicodeDecodeError as err:
-                    logger.error("Cannot decode configuration file '{}': {}".format(config_file, err))
+                    logger.error("Can not read configuration file '{}': {}".format(config_file, err))
                     sys.exit(1)
                 # Save the loaded configuration file path (issue #374)
                 self._loaded_config_file = config_file
@@ -160,7 +172,8 @@ class Config(object):
                              [str(iowait_bottleneck - (iowait_bottleneck * 0.20)),
                               str(iowait_bottleneck - (iowait_bottleneck * 0.10)),
                               str(iowait_bottleneck)])
-        ctx_switches_bottleneck = 56000 / multiprocessing.cpu_count()
+        # Context switches bottleneck identification #1212
+        ctx_switches_bottleneck = (500000 * 0.10) * multiprocessing.cpu_count()
         self.set_default_cwc('cpu', 'ctx_switches',
                              [str(ctx_switches_bottleneck - (ctx_switches_bottleneck * 0.20)),
                               str(ctx_switches_bottleneck - (ctx_switches_bottleneck * 0.10)),
@@ -249,17 +262,37 @@ class Config(object):
         self.set_default(section, header + 'warning', cwc[1])
         self.set_default(section, header + 'critical', cwc[2])
 
-    def set_default(self, section, option, default):
+    def set_default(self, section, option,
+                    default):
         """If the option did not exist, create a default value."""
         if not self.parser.has_option(section, option):
             self.parser.set(section, option, default)
 
-    def get_value(self, section, option, default=None):
-        """Get the value of an option, if it exists."""
+    def get_value(self, section, option,
+                  default=None):
+        """Get the value of an option, if it exists.
+
+        If it did not exist, then return de default value.
+
+        It allows user to define dynamic configuration key (see issue#1204)
+        Dynamic vlaue should starts and end with the ` char
+        Example: prefix=`hostname`
+        """
+        ret = default
         try:
-            return self.parser.get(section, option)
+            ret = self.parser.get(section, option)
         except NoOptionError:
-            return default
+            pass
+
+        # Search a substring `foo` and replace it by the result of its exec
+        if ret is not None:
+            try:
+                match = self.re_pattern.findall(ret)
+                for m in match:
+                    ret = ret.replace(m, system_exec(m[1:-1]))
+            except TypeError:
+                pass
+        return ret
 
     def get_int_value(self, section, option, default=0):
         """Get the int value of an option, if it exists."""
