@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,7 @@
 
 import sys
 
+import curses
 from glances.outputs.glances_curses import _GlancesCurses
 
 from glances.logger import logger
@@ -28,11 +29,10 @@ from glances.timer import Timer
 
 
 class GlancesCursesBrowser(_GlancesCurses):
-
     """Class for the Glances curse client browser."""
 
     def __init__(self, args=None):
-        # Init the father class
+        """Init the father class."""
         super(GlancesCursesBrowser, self).__init__(args=args)
 
         _colors_list = {
@@ -77,21 +77,21 @@ class GlancesCursesBrowser(_GlancesCurses):
         """Set the cursor position."""
         self.cursor_position = position
 
-    def cursor_up(self, servers_list):
+    def cursor_up(self, stats):
         """Set the cursor to position N-1 in the list."""
         if self.cursor_position > 0:
             self.cursor_position -= 1
         else:
-            self.cursor_position = len(servers_list) - 1
+            self.cursor_position = len(stats) - 1
 
-    def cursor_down(self, servers_list):
+    def cursor_down(self, stats):
         """Set the cursor to position N-1 in the list."""
-        if self.cursor_position < len(servers_list) - 1:
+        if self.cursor_position < len(stats) - 1:
             self.cursor_position += 1
         else:
             self.cursor_position = 0
 
-    def __catch_key(self, servers_list):
+    def __catch_key(self, stats):
         # Catch the browser pressed key
         self.pressedkey = self.get_key(self.term_window)
 
@@ -106,55 +106,61 @@ class GlancesCursesBrowser(_GlancesCurses):
             sys.exit(0)
         elif self.pressedkey == 10:
             # 'ENTER' > Run Glances on the selected server
-            logger.debug("Server number {} selected".format(self.cursor + 1))
             self.active_server = self.cursor
-        elif self.pressedkey == 65:
+            logger.debug("Server {}/{} selected".format(self.cursor + 1, len(stats)))
+        elif self.pressedkey == curses.KEY_UP or self.pressedkey == 65:
             # 'UP' > Up in the server list
-            self.cursor_up(servers_list)
-        elif self.pressedkey == 66:
+            self.cursor_up(stats)
+            logger.debug("Server {}/{} selected".format(self.cursor + 1, len(stats)))
+        elif self.pressedkey == curses.KEY_DOWN or self.pressedkey == 66:
             # 'DOWN' > Down in the server list
-            self.cursor_down(servers_list)
+            self.cursor_down(stats)
+            logger.debug("Server {}/{} selected".format(self.cursor + 1, len(stats)))
 
         # Return the key code
         return self.pressedkey
 
-    def update(self, servers_list):
+    def update(self,
+               stats,
+               duration=3,
+               cs_status=None,
+               return_to_browser=False):
         """Update the servers' list screen.
 
         Wait for __refresh_time sec / catch key every 100 ms.
 
-        servers_list: Dict of dict with servers stats
+        stats: Dict of dict with servers stats
         """
         # Flush display
-        logger.debug('Servers list: {}'.format(servers_list))
-        self.flush(servers_list)
+        logger.debug('Servers list: {}'.format(stats))
+        self.flush(stats)
 
         # Wait
         exitkey = False
         countdown = Timer(self.__refresh_time)
         while not countdown.finished() and not exitkey:
             # Getkey
-            pressedkey = self.__catch_key(servers_list)
+            pressedkey = self.__catch_key(stats)
             # Is it an exit or select server key ?
             exitkey = (
                 pressedkey == ord('\x1b') or pressedkey == ord('q') or pressedkey == 10)
             if not exitkey and pressedkey > -1:
                 # Redraw display
-                self.flush(servers_list)
+                self.flush(stats)
             # Wait 100ms...
             self.wait()
 
         return self.active_server
 
-    def flush(self, servers_list):
+    def flush(self, stats):
         """Update the servers' list screen.
 
-        servers_list: List of dict with servers stats
+        stats: List of dict with servers stats
         """
         self.erase()
-        self.display(servers_list)
+        self.display(stats)
 
-    def display(self, servers_list):
+    def display(self, stats, cs_status=None):
         """Display the servers list.
 
         Return:
@@ -167,30 +173,38 @@ class GlancesCursesBrowser(_GlancesCurses):
         # Get the current screen size
         screen_x = self.screen.getmaxyx()[1]
         screen_y = self.screen.getmaxyx()[0]
+        stats_max = screen_y - 3
+        stats_len = len(stats)
 
         # Init position
         x = 0
         y = 0
 
         # Display top header
-        if len(servers_list) == 0:
+        if stats_len == 0:
             if self.first_scan and not self.args.disable_autodiscover:
                 msg = 'Glances is scanning your network. Please wait...'
                 self.first_scan = False
             else:
                 msg = 'No Glances server available'
-        elif len(servers_list) == 1:
+        elif len(stats) == 1:
             msg = 'One Glances server available'
         else:
-            msg = '{} Glances servers available'.format(len(servers_list))
+            msg = '{} Glances servers available'.format(stats_len)
         if self.args.disable_autodiscover:
-            msg += ' ' + '(auto discover is disabled)'
-        self.term_window.addnstr(y, x,
-                                 msg,
-                                 screen_x - x,
-                                 self.colors_list['TITLE'])
+            msg += ' (auto discover is disabled)'
+        if screen_y > 1:
+            self.term_window.addnstr(y, x,
+                                     msg,
+                                     screen_x - x,
+                                     self.colors_list['TITLE'])
+        if stats_len > stats_max and screen_y > 2:
+            msg = 'Warning: Only {} servers will be displayed (please increase your terminal size)'.format(stats_max)
+            self.term_window.addnstr(y + 1, x,
+                                     msg,
+                                     screen_x - x)
 
-        if len(servers_list) == 0:
+        if stats_len == 0:
             return False
 
         # Display the Glances server list
@@ -224,13 +238,16 @@ class GlancesCursesBrowser(_GlancesCurses):
 
         # If a servers has been deleted from the list...
         # ... and if the cursor is in the latest position
-        if self.cursor > len(servers_list) - 1:
+        if self.cursor > len(stats) - 1:
             # Set the cursor position to the latest item
-            self.cursor = len(servers_list) - 1
+            self.cursor = len(stats) - 1
 
         # Display table
         line = 0
-        for v in servers_list:
+        for v in stats:
+            # Limit the number of displayed server (see issue #1256)
+            if line >= stats_max:
+                continue
             # Get server stats
             server_stat = {}
             for c in column_def:
