@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +25,6 @@ import psutil
 
 
 class Plugin(GlancesPlugin):
-
     """Glances CPU core plugin.
 
     Get stats about CPU core number.
@@ -41,32 +40,25 @@ class Plugin(GlancesPlugin):
         # The core number is displayed by the load plugin
         self.display_curse = False
 
-        # Init the stat
-        self.reset()
-
-    def reset(self):
-        """Reset/init the stat using the input method."""
-        self.stats = {}
-
     def update(self):
         """Update core stats.
 
         Stats is a dict (with both physical and log cpu number) instead of a integer.
         """
-        # Reset the stats
-        self.reset()
+        # Init new stats
+        stats = self.get_init_value()
 
         if self.input_method == 'local':
             # Update stats using the standard system lib
 
-            # The PSUtil 2.0 include psutil.cpu_count() and psutil.cpu_count(logical=False)
+            # The psutil 2.0 include psutil.cpu_count() and psutil.cpu_count(logical=False)
             # Return a dict with:
             # - phys: physical cores only (hyper thread CPUs are excluded)
             # - log: logical CPUs in the system
             # Return None if undefine
             try:
-                self.stats["phys"] = psutil.cpu_count(logical=False)
-                self.stats["log"] = psutil.cpu_count()
+                stats["phys"] = psutil.cpu_count(logical=False)
+                stats["log"] = psutil.cpu_count()
             except NameError:
                 self.reset()
 
@@ -75,4 +67,7 @@ class Plugin(GlancesPlugin):
             # http://stackoverflow.com/questions/5662467/how-to-find-out-the-number-of-cpus-using-snmp
             pass
 
+        # Update the stats
+        self.stats = stats
+
         return self.stats
