@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2015 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -35,20 +35,15 @@ snmp_oid = {'min1': '1.3.6.1.4.1.2021.10.1.3.1',
 
 # Define the history items list
 # All items in this list will be historised if the --enable-history tag is set
-# 'color' define the graph color in #RGB format
 items_history_list = [{'name': 'min1',
-                       'description': '1 minute load',
-                       'color': '#0000FF'},
+                       'description': '1 minute load'},
                       {'name': 'min5',
-                       'description': '5 minutes load',
-                       'color': '#0000AA'},
+                       'description': '5 minutes load'},
                       {'name': 'min15',
-                       'description': '15 minutes load',
-                       'color': '#000044'}]
+                       'description': '15 minutes load'}]
 
 
 class Plugin(GlancesPlugin):
-
     """Glances load plugin.
 
     stats is a dict
@@ -56,29 +51,24 @@ class Plugin(GlancesPlugin):
 
     def __init__(self, args=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args, items_history_list=items_history_list)
+        super(Plugin, self).__init__(args=args,
+                                     items_history_list=items_history_list)
 
         # We want to display the stat in the curse interface
         self.display_curse = True
 
-        # Init stats
-        self.reset()
-
         # Call CorePlugin in order to display the core number
         try:
             self.nb_log_core = CorePlugin(args=self.args).update()["log"]
         except Exception:
             self.nb_log_core = 1
 
-    def reset(self):
-        """Reset/init the stats."""
-        self.stats = {}
-
+    @GlancesPlugin._check_decorator
     @GlancesPlugin._log_result_decorator
     def update(self):
         """Update load stats."""
-        # Reset stats
-        self.reset()
+        # Init new stats
+        stats = self.get_init_value()
 
         if self.input_method == 'local':
             # Update stats using the standard system lib
@@ -87,32 +77,29 @@ class Plugin(GlancesPlugin):
             try:
                 load = os.getloadavg()
             except (OSError, AttributeError):
-                self.stats = {}
+                stats = self.get_init_value()
             else:
-                self.stats = {'min1': load[0],
-                              'min5': load[1],
-                              'min15': load[2],
-                              'cpucore': self.nb_log_core}
+                stats = {'min1': load[0],
+                         'min5': load[1],
+                         'min15': load[2],
+                         'cpucore': self.nb_log_core}
         elif self.input_method == 'snmp':
             # Update stats using SNMP
-            self.stats = self.get_stats_snmp(snmp_oid=snmp_oid)
+            stats = self.get_stats_snmp(snmp_oid=snmp_oid)
 
-            if self.stats['min1'] == '':
-                self.reset()
-                return self.stats
+            if stats['min1'] == '':
+                stats = self.get_init_value()
+                return stats
 
             # Python 3 return a dict like:
             # {'min1': "b'0.08'", 'min5': "b'0.12'", 'min15': "b'0.15'"}
-            for k, v in iteritems(self.stats):
-                self.stats[k] = float(v)
+            for k, v in iteritems(stats):
+                stats[k] = float(v)
 
-            self.stats['cpucore'] = self.nb_log_core
+            stats['cpucore'] = self.nb_log_core
 
-        # Update the history list
-        self.update_stats_history()
-
-        # Update the view
-        self.update_views()
+        # Update the stats
+        self.stats = stats
 
         return self.stats
 
@@ -131,13 +118,13 @@ class Plugin(GlancesPlugin):
             # try/except mandatory for Windows compatibility (no load stats)
             pass
 
-    def msg_curse(self, args=None):
+    def msg_curse(self, args=None, max_width=None):
         """Return the dict to display in the curse interface."""
         # Init the return message
         ret = []
 
         # Only process if stats exist, not empty (issue #871) and plugin not disabled
-        if not self.stats or (self.stats == {}) or args.disable_load:
+        if not self.stats or (self.stats == {}) or self.is_disable():
             return ret
 
         # Build the string message
