@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -26,12 +26,6 @@ import socket
 import time
 import numbers
 
-try:
-    import requests
-    requests_tag = True
-except ImportError:
-    requests_tag = False
-
 from glances.globals import WINDOWS, MACOS, BSD
 from glances.ports_list import GlancesPortsList
 from glances.web_list import GlancesWebList
@@ -40,14 +34,21 @@ from glances.compat import bool_type
 from glances.logger import logger
 from glances.plugins.glances_plugin import GlancesPlugin
 
+try:
+    import requests
+    requests_tag = True
+except ImportError as e:
+    requests_tag = False
+    logger.warning("Missing Python Lib ({}), Ports plugin is limited to port scanning".format(e))
+
 
 class Plugin(GlancesPlugin):
-
     """Glances ports scanner plugin."""
 
     def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args)
+        super(Plugin, self).__init__(args=args,
+                                     stats_init_value=[])
         self.args = args
         self.config = config
 
@@ -64,20 +65,15 @@ class Plugin(GlancesPlugin):
         self._thread = None
 
     def exit(self):
-        """Overwrite the exit method to close threads"""
+        """Overwrite the exit method to close threads."""
         if self._thread is not None:
             self._thread.stop()
         # Call the father class
         super(Plugin, self).exit()
 
-    def reset(self):
-        """Reset/init the stats."""
-        self.stats = []
-
     @GlancesPlugin._log_result_decorator
     def update(self):
         """Update the ports list."""
-
         if self.input_method == 'local':
             # Only refresh:
             # * if there is not other scanning thread
@@ -103,7 +99,6 @@ class Plugin(GlancesPlugin):
 
     def get_ports_alert(self, port, header="", log=False):
         """Return the alert status relative to the port scan return value."""
-
         if port['status'] is None:
             return 'CAREFUL'
         elif port['status'] == 0:
@@ -117,7 +112,6 @@ class Plugin(GlancesPlugin):
 
     def get_web_alert(self, web, header="", log=False):
         """Return the alert status relative to the web/url scan return value."""
-
         if web['status'] is None:
             return 'CAREFUL'
         elif web['status'] not in [200, 301, 302]:
@@ -136,6 +130,9 @@ class Plugin(GlancesPlugin):
         if not self.stats or args.disable_ports:
             return ret
 
+        # Max size for the interface name
+        name_max_width = max_width - 7
+
         # Build the string message
         for p in self.stats:
             if 'host' in p:
@@ -151,13 +148,15 @@ class Plugin(GlancesPlugin):
                     # Convert second to ms
                     status = '{0:.0f}ms'.format(p['status'] * 1000.0)
 
-                msg = '{:14.14} '.format(p['description'])
+                msg = '{:{width}}'.format(p['description'][0:name_max_width],
+                                          width=name_max_width)
                 ret.append(self.curse_add_line(msg))
-                msg = '{:>8}'.format(status)
+                msg = '{:>9}'.format(status)
                 ret.append(self.curse_add_line(msg, self.get_ports_alert(p)))
                 ret.append(self.curse_new_line())
             elif 'url' in p:
-                msg = '{:14.14} '.format(p['description'])
+                msg = '{:{width}}'.format(p['description'][0:name_max_width],
+                                          width=name_max_width)
                 ret.append(self.curse_add_line(msg))
                 if isinstance(p['status'], numbers.Number):
                     status = 'Code {}'.format(p['status'])
@@ -165,7 +164,7 @@ class Plugin(GlancesPlugin):
                     status = 'Scanning'
                 else:
                     status = p['status']
-                msg = '{:>8}'.format(status)
+                msg = '{:>9}'.format(status)
                 ret.append(self.curse_add_line(msg, self.get_web_alert(p)))
                 ret.append(self.curse_new_line())
 
@@ -186,7 +185,7 @@ class ThreadScanner(threading.Thread):
     """
 
     def __init__(self, stats):
-        """Init the class"""
+        """Init the class."""
         logger.debug("ports plugin - Create thread for scan list {}".format(stats))
         super(ThreadScanner, self).__init__()
         # Event needed to stop properly the thread
@@ -197,9 +196,10 @@ class ThreadScanner(threading.Thread):
         self.plugin_name = "ports"
 
     def run(self):
-        """Function called to grab stats.
-        Infinite loop, should be stopped by calling the stop() method"""
+        """Grab the stats.
 
+        Infinite loop, should be stopped by calling the stop() method.
+        """
         for p in self._stats:
             # End of the thread has been asked
             if self.stopped():
@@ -216,25 +216,25 @@ class ThreadScanner(threading.Thread):
 
     @property
     def stats(self):
-        """Stats getter"""
+        """Stats getter."""
         return self._stats
 
     @stats.setter
     def stats(self, value):
-        """Stats setter"""
+        """Stats setter."""
         self._stats = value
 
     def stop(self, timeout=None):
-        """Stop the thread"""
+        """Stop the thread."""
         logger.debug("ports plugin - Close thread for scan list {}".format(self._stats))
         self._stopper.set()
 
     def stopped(self):
-        """Return True is the thread is stopped"""
+        """Return True is the thread is stopped."""
         return self._stopper.isSet()
 
     def _web_scan(self, web):
-        """Scan the  Web/URL (dict) and update the status key"""
+        """Scan the  Web/URL (dict) and update the status key."""
         try:
             req = requests.head(web['url'],
                                 allow_redirects=True,
@@ -249,14 +249,14 @@ class ThreadScanner(threading.Thread):
         return web
 
     def _port_scan(self, port):
-        """Scan the port structure (dict) and update the status key"""
+        """Scan the port structure (dict) and update the status key."""
         if int(port['port']) == 0:
             return self._port_scan_icmp(port)
         else:
             return self._port_scan_tcp(port)
 
     def _resolv_name(self, hostname):
-        """Convert hostname to IP address"""
+        """Convert hostname to IP address."""
         ip = hostname
         try:
             ip = socket.gethostbyname(hostname)
@@ -265,7 +265,7 @@ class ThreadScanner(threading.Thread):
         return ip
 
     def _port_scan_icmp(self, port):
-        """Scan the (ICMP) port structure (dict) and update the status key"""
+        """Scan the (ICMP) port structure (dict) and update the status key."""
         ret = None
 
         # Create the ping command
@@ -305,7 +305,7 @@ class ThreadScanner(threading.Thread):
         return ret
 
     def _port_scan_tcp(self, port):
-        """Scan the (TCP) port structure (dict) and update the status key"""
+        """Scan the (TCP) port structure (dict) and update the status key."""
         ret = None
 
         # Create and configure the scanning socket
