@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2015 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -19,14 +19,17 @@
 
 """Manage the Glances standalone session."""
 
-from time import sleep
+import sys
+import time
 
 from glances.globals import WINDOWS
 from glances.logger import logger
 from glances.processes import glances_processes
 from glances.stats import GlancesStats
 from glances.outputs.glances_curses import GlancesCursesStandalone
+from glances.outputs.glances_stdout import GlancesStdout
 from glances.outdated import Outdated
+from glances.timer import Counter
 
 
 class GlancesStandalone(object):
@@ -34,6 +37,9 @@ class GlancesStandalone(object):
     """This class creates and manages the Glances standalone session."""
 
     def __init__(self, config=None, args=None):
+        self.config = config
+        self.args = args
+
         # Quiet mode
         self._quiet = args.quiet
         self.refresh_time = args.time
@@ -41,6 +47,12 @@ class GlancesStandalone(object):
         # Init stats
         self.stats = GlancesStats(config=config, args=args)
 
+        # Modules (plugins and exporters) are loaded at this point
+        # Glances can display the list if asked...
+        if args.modules_list:
+            self.display_modules_list()
+            sys.exit(0)
+
         # If process extended stats is disabled by user
         if not args.enable_process_extended:
             logger.debug("Extended stats for top process are disabled")
@@ -57,20 +69,17 @@ class GlancesStandalone(object):
             # Ignore kernel threads in process list
             glances_processes.disable_kernel_threads()
 
-        try:
-            if args.process_tree:
-                # Enable process tree view
-                glances_processes.enable_tree()
-        except AttributeError:
-            pass
-
         # Initial system informations update
         self.stats.update()
 
         if self.quiet:
-            logger.info("Quiet mode is ON: Nothing will be displayed")
+            logger.info("Quiet mode is ON, nothing will be displayed")
             # In quiet mode, nothing is displayed
             glances_processes.max_processes = 0
+        elif args.stdout:
+            logger.info("Stdout mode is ON, following stats will be displayed: {}".format(args.stdout))
+            # Init screen
+            self.screen = GlancesStdout(config=config, args=args)
         else:
             # Default number of processes to displayed is set to 50
             glances_processes.max_processes = 50
@@ -85,32 +94,52 @@ class GlancesStandalone(object):
     def quiet(self):
         return self._quiet
 
+    def display_modules_list(self):
+        """Display modules list"""
+        print("Plugins list: {}".format(
+            ', '.join(sorted(self.stats.getPluginsList(enable=False)))))
+        print("Exporters list: {}".format(
+            ', '.join(sorted(self.stats.getExportsList(enable=False)))))
+
     def __serve_forever(self):
-        """Main loop for the CLI."""
-        while True:
-            # Update system informations
-            self.stats.update()
+        """Main loop for the CLI.
 
-            if not self.quiet:
-                # Update the screen
-                self.screen.update(self.stats)
-            else:
-                # Wait...
-                sleep(self.refresh_time)
+        return True if we should continue (no exit key has been pressed)
+        """
+        # Start a counter used to compute the time needed for
+        # update and export the stats
+        counter = Counter()
 
-            # Export stats using export modules
-            self.stats.export(self.stats)
+        # Update stats
+        self.stats.update()
+        logger.debug('Stats updated in {} seconds'.format(counter.get()))
+
+        # Export stats
+        counter_export = Counter()
+        self.stats.export(self.stats)
+        logger.debug('Stats exported in {} seconds'.format(counter_export.get()))
+
+        # Display stats
+        # and wait refresh_time - counter
+        if not self.quiet:
+            # The update function return True if an exit key 'q' or 'ESC'
+            # has been pressed.
+            ret = not self.screen.update(self.stats,
+                                         duration=self.refresh_time - counter.get())
+        else:
+            # Nothing is displayed
+            # Break should be done via a signal (CTRL-C)
+            time.sleep(self.refresh_time - counter.get())
+            ret = True
+
+        return ret
 
     def serve_forever(self):
-        """Wrapper to the serve_forever function.
-
-        This function will restore the terminal to a sane state
-        before re-raising the exception and generating a traceback.
-        """
-        try:
-            return self.__serve_forever()
-        finally:
-            self.end()
+        """Wrapper to the serve_forever function."""
+        loop = True
+        while loop:
+            loop = self.__serve_forever()
+        self.end()
 
     def end(self):
         """End of the standalone CLI."""
@@ -120,7 +149,7 @@ class GlancesStandalone(object):
         # Exit from export modules
         self.stats.end()
 
-        # Check Glances version versus Pypi one
+        # Check Glances version versus PyPI one
         if self.outdated.is_outdated():
             print("You are using Glances version {}, however version {} is available.".format(
                 self.outdated.installed_version(), self.outdated.latest_version()))
